@echo off

set "vs_version=%~1"
set "build_type=%~2"
set "libs_type=%~3"
set "architecture=%~4"

if "%architecture%"=="x86" (
    set arch="Win32"
) else if "%architecture%"=="x64" (
    set arch="%architecture%"
) else (
    goto InvalidArguments
)

if not "%build_type%"=="Debug" if not "%build_type%"=="Release" goto InvalidArguments

if "%vs_version%"=="v140" (
    set generator="Visual Studio 14 2015"
) else if "%vs_version%"=="v141" (
    set generator="Visual Studio 15 2017"
) else if "%vs_version%"=="v142" (
    set generator="Visual Studio 16"
) else if "%vs_version%"=="v143" (
    set generator="Visual Studio 17"
) else (
    goto InvalidArguments
)

if exist build (
    echo Deleting the existing build folder...
    rmdir /s /q build
)

if "%libs_type%"=="staticMD" (
    set "cmake_command=cmake -B build -G %generator% -DCMAKE_BUILD_TYPE=%build_type% -A %arch% -T host=%architecture%"
) else if "%libs_type%"=="staticMT" (
    set "cmake_command=cmake -B build -G %generator% -DCMAKE_BUILD_TYPE=%build_type% -A %arch% -T host=%architecture%"
) else if "%libs_type%"=="shared" (
    set "cmake_command=cmake -B build -G %generator% -DCMAKE_BUILD_TYPE=%build_type% -A %arch% -T host=%architecture%"
) else (
    goto InvalidArguments
)

echo Running CMake...
echo %cmake_command%
%cmake_command% .

if %errorlevel% neq 0 (
    echo CMake configuration failed.
    exit /b 1
)

echo Building the project...
cmake --build build --config %build_type%

if %errorlevel% neq 0 (
    echo Build failed.
    exit /b 1
)

echo Build completed successfully.
exit /b 0

:InvalidArguments
echo Invalid arguments! Usage: build.bat [PlatformToolset] [BuildType] [Configuration] [Architecture]
echo Supported platform toolsets: v140, v141, v142 and v143.
echo Supported build types: Debug or Release.
echo Supported configurations: staticMD, staticMT, or shared.
echo Supported architectures: x86 or x64.
exit /b 1
